@@ -23,10 +23,10 @@
  * problem since this code was originally created in Europe and India.
  */
 
-/* Need a type that should be 32 bits. 64 also works but wastes space. */
-typedef unsigned int u32_T;	/* int is at least 32 bits */
+// Need a type that should be 32 bits. 64 also works but wastes space.
+typedef unsigned int u32_T;	// int is at least 32 bits
 
-/* The state of encryption, referenced by cryptstate_T. */
+// The state of encryption, referenced by cryptstate_T.
 typedef struct {
     u32_T keys[3];
 } zip_state_T;
@@ -151,4 +151,4 @@ crypt_zip_decode(
     }
 }
 
-#endif /* FEAT_CRYPT */
+#endif // FEAT_CRYPT
