@@ -1659,10 +1659,13 @@ win_update(win_T *wp)
 #endif
 	    )
     {
-	if (mod_top != 0 && wp->w_topline == mod_top)
+	if (mod_top != 0
+		&& wp->w_topline == mod_top
+		&& (!wp->w_lines[0].wl_valid
+		    || wp->w_topline == wp->w_lines[0].wl_lnum))
 	{
-	    // w_topline is the first changed line, the scrolling will be done
-	    // further down.
+	    // w_topline is the first changed line and window is not scrolled,
+	    // the scrolling from changed lines will be done further down.
 	}
 	else if (wp->w_lines[0].wl_valid
 		&& (wp->w_topline < wp->w_lines[0].wl_lnum
