@@ -757,6 +757,10 @@ set_indent(
     // Replace the line (unless undo fails).
     if (!(flags & SIN_UNDO) || u_savesub(curwin->w_cursor.lnum) == OK)
     {
+	colnr_T old_offset = (colnr_T)(p - oldline);
+	colnr_T new_offset = (colnr_T)(s - newline);
+
+	// this may free "newline"
 	ml_replace(curwin->w_cursor.lnum, newline, FALSE);
 	if (flags & SIN_CHANGED)
 	    changed_bytes(curwin->w_cursor.lnum, 0);
@@ -764,24 +768,24 @@ set_indent(
 	// Correct saved cursor position if it is in this line.
 	if (saved_cursor.lnum == curwin->w_cursor.lnum)
 	{
-	    if (saved_cursor.col >= (colnr_T)(p - oldline))
+	    if (saved_cursor.col >= old_offset)
 		// cursor was after the indent, adjust for the number of
 		// bytes added/removed
-		saved_cursor.col += ind_len - (colnr_T)(p - oldline);
-	    else if (saved_cursor.col >= (colnr_T)(s - newline))
+		saved_cursor.col += ind_len - old_offset;
+	    else if (saved_cursor.col >= new_offset)
 		// cursor was in the indent, and is now after it, put it back
 		// at the start of the indent (replacing spaces with TAB)
-		saved_cursor.col = (colnr_T)(s - newline);
+		saved_cursor.col = new_offset;
 	}
 #ifdef FEAT_PROP_POPUP
 	{
-	    int added = ind_len - (colnr_T)(p - oldline);
+	    int added = ind_len - old_offset;
 
 	    // When increasing indent this behaves like spaces were inserted at
 	    // the old indent, when decreasing indent it behaves like spaces
 	    // were deleted at the new indent.
 	    adjust_prop_columns(curwin->w_cursor.lnum,
-		 (colnr_T)(added > 0 ? (p - oldline) : ind_len), added, 0);
+			  added > 0 ? old_offset : (colnr_T)ind_len, added, 0);
 	}
 #endif
 	retval = TRUE;
@@ -875,9 +879,9 @@ briopt_check(win_T *wp)
 	    ++p;
     }
 
-    wp->w_p_brishift = bri_shift;
-    wp->w_p_brimin   = bri_min;
-    wp->w_p_brisbr   = bri_sbr;
+    wp->w_briopt_shift = bri_shift;
+    wp->w_briopt_min   = bri_min;
+    wp->w_briopt_sbr   = bri_sbr;
 
     return OK;
 }
@@ -927,10 +931,10 @@ get_breakindent_win(
 				     (int)wp->w_buffer->b_p_ts, wp->w_p_list);
 # endif
     }
-    bri = prev_indent + wp->w_p_brishift;
+    bri = prev_indent + wp->w_briopt_shift;
 
     // indent minus the length of the showbreak string
-    if (wp->w_p_brisbr)
+    if (wp->w_briopt_sbr)
 	bri -= vim_strsize(get_showbreak_value(wp));
 
     // Add offset for number column, if 'n' is in 'cpoptions'
@@ -941,9 +945,9 @@ get_breakindent_win(
 	bri = 0;
     // always leave at least bri_min characters on the left,
     // if text width is sufficient
-    else if (bri > eff_wwidth - wp->w_p_brimin)
-	bri = (eff_wwidth - wp->w_p_brimin < 0)
-			    ? 0 : eff_wwidth - wp->w_p_brimin;
+    else if (bri > eff_wwidth - wp->w_briopt_min)
+	bri = (eff_wwidth - wp->w_briopt_min < 0)
+					   ? 0 : eff_wwidth - wp->w_briopt_min;
 
     return bri;
 }
@@ -1760,7 +1764,7 @@ get_expr_indent(void)
     set_vim_var_nr(VV_LNUM, curwin->w_cursor.lnum);
     if (use_sandbox)
 	++sandbox;
-    ++textlock;
+    ++textwinlock;
 
     // Need to make a copy, the 'indentexpr' option could be changed while
     // evaluating it.
@@ -1773,7 +1777,7 @@ get_expr_indent(void)
 
     if (use_sandbox)
 	--sandbox;
-    --textlock;
+    --textwinlock;
 
     // Restore the cursor position so that 'indentexpr' doesn't need to.
     // Pretend to be in Insert mode, allow cursor past end of line for "o"
